library getrevid_wharf; // version: 1999/05/15
//
//     _   _   _         _ _                    __  ____    _____     __  ____
//    //_ //_ //_ __    ////_ _ _ _ _ _ _ _ _  /__\/ __/\  / / _ \   /__\/ __/
//   /_ // _// _//_ \o ////////////////////// //_// __/\ \/ /||_||  //_// __/
//  //// \_/ \_// __/o/////____//____//____/o/___/\__/  \__/ \___/o/___/___/
//             /_/
//
// ----------------------------------------------------------------------------
// getrevid_wharf_test is an example how to support the revision id's which
// are gathered by the '!About detailed'-Dialog.
// ----------------------------------------------------------------------------
//
// history:
// 1999/05/16 - murphy@dev0.de
// - first start of writing. test runs ok!
//

uses
  SysUtils,
  Classes,
  Windows,
  Messages,
  Graphics,
  lsapi     in '..\..\units\lsapi.pas',
  wharfdata in '..\..\units\wharfdata.pas';

var
  Revision:array [0..20] of char = 'Revision: 0.01'#0#0; // Our Version
  revId:array[0..55] of char = 'Id: getrevid_wharf.pas,v 0.01 1999/05/16 17:06 murphy'#0#0; // The Full ID.

var
  szAppName:array [0..15] of char = 'getrevid_wharf'#0#0;

var
  hMainWnd,                  // main window handle
  parent:HWND;               // parent window
  wharf:wharfDataType;       // Setup data passed to the DLL
  wndSize:LongInt;           // Size of the wharf client

procedure PaintWnd(Wnd:HWnd);  // out paint procedure
var
  ps:TPaintStruct;
  fHdc:HDC;
  Canvas:TCanvas;
begin
  fHDC:=BeginPaint(Wnd,ps);
  Canvas:=TCanvas.Create;
  Canvas.Handle:=fHDC;
  Canvas.Font.Style:=[];
  Canvas.TextOut(10,10,'getrevid');
  EndPaint(Wnd,ps);
  Canvas.Free;
end;

function WndProc(Wnd:HWND;Msg:uint;wParam:wParam;lParam:lParam):LRESULT; stdcall;
var
  r:TRect;
  buf:PChar;
begin
  case Msg of
    WM_CREATE:     // window create ok
      result:=0;
    WM_ERASEBKGND: // don't erase background
      result:=0;
    WM_KEYDOWN,    // forward keyboard messages to parent window
    WM_KEYUP:
      begin
        PostMessage(parent,msg,wParam,lParam);
        result:=0;
      end;
    WM_PAINT:      // paint your wharf here...
      begin
        PaintWnd(Wnd);
        result:=0;
      end;
    // mouse messages are here to ensure we have a good popup menu behaviour. you may insert
    // your own custom actions
    WM_LBUTTONUP:
      begin
        Result:=0;
      end;
    WM_RBUTTONUP:  // popup popupmenu :-)
      begin
        GetWindowRect(Wnd,r);
        PostMessage(GetLiteStepWnd,LM_POPUP,r.Top+HIWORD(lParam),r.Left+LOWORD(lParam));
        result:=0;
      end;
    WM_RBUTTONDOWN,// hide popupmenu
    WM_LBUTTONDOWN,
    WM_MBUTTONDOWN:
      begin
        PostMessage(GetLiteStepWnd,LM_HIDEPOPUP,0,0);
        result:=0;
      end;
    LM_GETREVID:  // Get our revision for '!About detailed'
      begin
        buf:=PChar(lParam);
        if (wParam=0) then
        begin
          StrCopy(buf,'getrevid_whard.dll: ');
          StrCat(buf,Revision);
        end
        else if (wParam=1) then
        begin
          StrCopy(buf,revId);
        end
        else
        begin
          StrCopy(buf,'');
        end;
        result:=StrLen(buf);
      end;
  else             // default window proc
    result:=DefWindowProc(Wnd,msg,wParam,lParam);
  end;
End;

function initWharfModule(ParentWnd:HWND;dllInst:Longint;wd:WharfDataTypePtr):longInt; cdecl;
var
  wc:TWndClass;
  Msgs:array [0..1] of longint;
begin
  parent:=ParentWnd; // Save parent window

  // Duplicate wharfData since the one we get will be destroyed
  move(wd^,wharf,sizeof(wharfDataType));
  if wharf.BorderSize<0 then
    wharf.BorderSize:=0;
  wndSize:=64-(wharf.borderSize*2);

  // Register the Window class
  FillChar(wc,sizeof(wc), #0);
  wc.lpfnWndProc:=@WndProc;       // our window procedure
  wc.hInstance:=DllInst;          // hInstance of DLL
  wc.lpszClassName:=szAppName;    // our window class name

  if (RegisterClass(wc)=0) then   // when error then msgbox and exit
  begin
    MessageBox(parent,'error registering window class',szAppName,MB_OK);
    result:=1;
    exit;
  end;

  // Create out window on the wharf!
  hMainWnd:=CreateWindowEx(
    WS_EX_TRANSPARENT,                          // exstyles
    szAppName,                                  // our window class name
    szAppName,                                  // use description for a window title
    WS_CHILD,                                   // window style
    wharf.borderSize, wharf.borderSize,         // position
    wndSize,wndSize,                            // width & height of window
    parent,                                     // parent window (litestep wharf window)
    0,                                          // no menu
    dllInst,                                    // hInstance of DLL
    nil);                                       // no window creation data

  if (hMainWnd=0) then            // when error then msgbox and exit
  begin
    MessageBox(parent,'error creating window',szAppName,MB_OK);
    result:=1;
    exit;
  end;

  // Set normal cursor
  SetCursor(LoadCursor(0,MAKEINTRESOURCE(IDC_ARROW)));

  // DO NOT REMOVE! set magicDWord is required!
  // used by some modules & litestep internals
  SetWindowLong(hMainWnd,GWL_USERDATA,magicDWord);

  // show the window
  ShowWindow(hMainWnd,SW_SHOWNORMAL);

  // register our windowmessages!
  Msgs[0] := LM_GETREVID;
  Msgs[1] := 0;
  SendMessage(GetLitestepWnd, LM_REGISTERMESSAGE, hMainWnd, Integer(@Msgs));

  result:=0;  // exit with ok!
end;

procedure quitWharfModule(dllInst:Longint); cdecl;
var
  Msgs:array [0..1] of longint;
begin
  // unregister our windowmessages!
  Msgs[0] := LM_GETREVID;
  Msgs[1] := 0;
  SendMessage(GetLitestepWnd, LM_UNREGISTERMESSAGE, hMainWnd, Integer(@Msgs));
  
  DestroyWindow(hMainWnd);                // delete our window
  UnregisterClass(szAppName,dllInst);     // unregister window class
end;

exports       // these are the exports
  initWharfModule index 1,
  quitWharfModule index 2;

begin
  // here is no code!
end.

