library mixed_module; // version: 1999/05/16
//
//     _   _   _         _ _                    __  ____    _____     __  ____
//    //_ //_ //_ __    ////_ _ _ _ _ _ _ _ _  /__\/ __/\  / / _ \   /__\/ __/
//   /_ // _// _//_ \o ////////////////////// //_// __/\ \/ /||_||  //_// __/
//  //// \_/ \_// __/o/////____//____//____/o/___/\__/  \__/ \___/o/___/___/
//             /_/
//
// ----------------------------------------------------------------------------
// mixed_module is an example how to use the lsdevkit for delphi. it shoes how
// to program a wharf that can be loaded on desktop and on wharf...
// ----------------------------------------------------------------------------
//
// history:
// 1999/05/16 - murphy@dev0.de
// - first start of writing. test ok!
//

uses
  SysUtils,
  Classes,
  Windows,
  Messages,
  Graphics,
  lsapi in '..\..\units\lsapi.pas',
  wharfdata in '..\..\units\wharfdata.pas',
  lstools in '..\..\units\lstools.pas';

var
  szAppName:array [0..13] of char = 'mixed_module'#0#0;

var
  hMainWnd,                  // main window handle
  parent:HWND;               // parent window
  wharf:wharfDataType;       // Setup data passed to the DLL
  wndSize:LongInt;           // Size of the wharf client
  DockOnWharf:Boolean;       // docked on wharf or desktop?

procedure PaintWnd(Wnd:HWnd);  // out paint procedure
var
  ps:TPaintStruct;
  fHdc:HDC;
  Canvas:TCanvas;
begin
  fHDC:=BeginPaint(Wnd,ps);
  Canvas:=TCanvas.Create;
  Canvas.Handle:=fHDC;
  Canvas.Font.Style:=[];
  Canvas.Brush.Color:=clRed;
  Canvas.Pen.Color:=clYellow;
  Canvas.Rectangle(0,0,64,64);
  Canvas.TextOut(10,10,'test');
  EndPaint(Wnd,ps);
  Canvas.Free;
end;

function WndProc(Wnd:HWND;Msg:uint;wParam:wParam;lParam:lParam):LRESULT; stdcall;
var
  r:TRect;
  pwp:^TWINDOWPOS;
begin
  case Msg of
    WM_CREATE:     // window create ok
      result:=0;
    WM_ERASEBKGND: // don't erase background
      result:=0;
    WM_KEYDOWN,    // forward keyboard messages to parent window
    WM_KEYUP:
      begin
        PostMessage(parent,msg,wParam,lParam);
        result:=0;
      end;
    WM_PAINT:      // paint your wharf here...
      begin
        PaintWnd(Wnd);
        result:=0;
      end;
    // mouse messages are here to ensure we have a good popup menu behaviour. you may insert
    // your own custom actions
    WM_LBUTTONUP:
      begin
        Result:=0;
      end;
    WM_RBUTTONUP:  // popup popupmenu :-)
      begin
        GetWindowRect(Wnd,r);
        PostMessage(GetLiteStepWnd,LM_POPUP,r.Top+HIWORD(lParam),r.Left+LOWORD(lParam));
        result:=0;
      end;
    WM_RBUTTONDOWN,// hide popupmenu
    WM_LBUTTONDOWN,
    WM_MBUTTONDOWN:
      begin
        PostMessage(GetLiteStepWnd,LM_HIDEPOPUP,0,0);
        result:=0;
      end;
    WM_WINDOWPOSCHANGING:
      if not DockOnWharf then
      begin
        pwp:=Pointer(lParam);
        //if( !(pwp->flags & SWP_NOZORDER) )
        pwp.hwndInsertAfter:=HWND_BOTTOM;
        result:=0;
      end
      else
      begin
        result:=DefWindowProc(Wnd,msg,wParam,lParam);
      end;
  else
    result:=DefWindowProc(Wnd,msg,wParam,lParam);
  end;
End;

function initModuleEx(ParentWnd:HWND;dllInst:Longint;szPath:PCHAR):longint; cdecl;
var
  wc:TWndClass;
begin
  parent:=ParentWnd; // Save parent window

  // Register the Window class
  FillChar(wc,sizeof(wc), #0);
  wc.lpfnWndProc:=@WndProc;       // our window procedure
  wc.hInstance:=DllInst;          // hInstance of DLL
  wc.lpszClassName:=szAppName;    // our window class name

  if (RegisterClass(wc)=0) then   // when error then msgbox and exit
  begin
    MessageBox(parent,'error registering window class',szAppName,MB_OK);
    result:=1;
    exit;
  end;

  if (DockOnWharf) then
  begin
    // Create out window on the wharf!
    hMainWnd:=CreateWindowEx(
      WS_EX_TRANSPARENT,                          // exstyles
      szAppName,                                  // our window class name
      szAppName,                                  // use description for a window title
      WS_CHILD,                                   // window style
      wharf.borderSize, wharf.borderSize,         // position
      wndSize,wndSize,                            // width & height of window
      parent,                                     // parent window (litestep wharf window)
      0,                                          // no menu
      dllInst,                                    // hInstance of DLL
      nil);                                       // no window creation data
  end
  else
  begin
    // Create out window on the wharf!
    wndSize:=64;
    hMainWnd:=CreateWindowEx(
      WS_EX_TOOLWINDOW or WS_EX_TRANSPARENT,
      szAppName,
      szAppName,
      WS_POPUP or WS_VISIBLE,
      100,100,
      wndSize,wndSize,
      parent,
      0,
      dllInst,
      nil);
    SetParent(hMainWnd,GetLitestepDesktop);
  end;

  if (hMainWnd=0) then            // when error then msgbox and exit
  begin
    MessageBox(parent,'error creating window',szAppName,MB_OK);
    result:=1;
    exit;
  end;

  // Set normal cursor
  SetCursor(LoadCursor(0,MAKEINTRESOURCE(IDC_ARROW)));

  // DO NOT REMOVE! set magicDWord is required!
  // used by some modules & litestep internals
  SetWindowLong(hMainWnd,GWL_USERDATA,magicDWord);

  // show the window
  ShowWindow(hMainWnd,SW_SHOWNORMAL);

  result:=0;  // exit with ok!
end;

function initModule(ParentWnd:HWND;dllInst:Longint;wd:WharfDataTypePtr):longint; cdecl;
begin
  result:=initModuleEx(ParentWnd,dllInst,wd.lsPath);
end;

function initWharfModule(ParentWnd:HWND;dllInst:Longint;wd:WharfDataTypePtr):longInt; cdecl;
begin
  DockOnWharf:=true;

  // Duplicate wharfData since the one we get will be destroyed
  move(wd^,wharf,sizeof(wharfDataType));
  if wharf.BorderSize<0 then
    wharf.BorderSize:=0;
  wndSize:=64-(wharf.borderSize*2);
  
  result:=initModuleEx(ParentWnd,dllInst,wd.lsPath);
end;

procedure quitModule(dllInst:Longint); cdecl;
begin
  DestroyWindow(hMainWnd);                // delete our window
  UnregisterClass(szAppName,dllInst);     // unregister window class
end;

procedure quitWharfModule(dllInst:Longint); cdecl;
begin
  quitModule(dllInst);
end;

exports       // these are the exports
  initWharfModule index 1,
  quitWharfModule index 2,
  initModule      index 3,
  initModuleEx    index 4,
  quitModule      index 5;

begin
  // here is no code!
end.

